// OracleColumn.h
// Copyright (c) 1994, NeXT Computer, Inc.  All rights reserved.

#import "ociapr.h"
#import <eoaccess/eoaccess.h>

// This class is used internally by the Oracle adaptor to maintain the state
// of the various column buffers which are needed to fetch data.

// These are the codes for internal (in the server) Oracle data types.
// OracleServerTypes are mapped by the adaptor to the "external type" string
// of an EOAttribute (unfortunately, Oracle and EOF have opposite ideas of
// what internal and external mean).

typedef enum { 
    OraVARCHAR2 =  1,
    OraNUMBER   =  2,
    OraLONG     =  8,
    OraROWID    = 11,
    OraDATE     = 12,
    OraRAW      = 23,
    OraLONGRAW  = 24,
    OraCHAR     = 96,
    OraMLSLABEL = 106
} OracleServerType;

// These are the codes for the external (in the client) Oracle data types.
// OracleClientTypes specify how OCI should convert the data from the rdbms
// into a form that can be consumed by the adaptor, or to describe the format
// of of data passed into OCI by the adaptor.  Not all of these are used by
// the current implementation, but are included here for completeness.

typedef enum {
    OciVARCHAR2    =  1,
    OciNUMBER      =  2,
    OciINTEGER     =  3,
    OciFLOAT       =  4,
    OciSTRING      =  5,
    OciVARNUM      =  6,
    OciPACKED      =  7,
    OciLONG        =  8,
    OciVARCHAR     =  9,
    OciROWID       = 11,
    OciDATE        = 12,
    OciVARRAW      = 15,
    OciRAW         = 23,
    OciLONGRAW     = 24,
    OciUNSIGNED    = 68,
    OciDISPLAY     = 91,
    OciLONGVARCHAR = 94,
    OciLONGVARRAW  = 95,
    OciCHAR        = 96,
    OciCHARZ       = 97,
    OciMLSLABEL    = 106
} OracleClientType;

@class OracleChannel;

// This is the abstract class from which all column types descend.  It
// implements the various class methods, but subclasses must implement all of
// the instance methods.

@interface OracleColumn:NSObject

+ (OracleServerType)oracleServerTypeForName:(NSString *)typeName;
    // Given an external type string from an attribute, this returns the type
    // code for that data type as understood by OCI.

+ formatAttribute:(EOAttribute *)attribute;
+ formatValue:value forAttribute:(EOAttribute *)attribute;
+ (BOOL)isValidQualifierType:(NSString *)typeName;
    // Methods called by the EO framework (through the adaptor) to generate
    // correct SQL.

+ primitiveValueForCustomValue:value attribute:(EOAttribute *)attribute;
    // Takes a custom value and extracts the primitive data type (NSString or
    // NSData depending on the external type of the attribute).  Does nothing
    // if the value is not a custom value.

+ (EOAttribute *)attributeWithName:(NSString *)name
    columnName:(NSString *)columnName externalType:(NSString *)dbType
    externalLength:(int)dbLength precision:(int)precision scale:(int)scale;
    // Builds an EOAttribute based on the standard set of information
    // available in Oracle v7 about database columns.  This is called from
    // attributeAtPosition:cursor: and by the describe routines.

+ (EOAttribute *)attributeAtPosition:(int)position cursor:(Cda_Def *)cda;
    // This uses odescr() to build an attribute for the item at position in
    // the select list.  It can only by called after oparse() and before
    // oexec().  Position indices start at 0.

- initWithAttribute:(EOAttribute *)attribute;
    // This is the designated initializer for all OracleColumns.

- (int)columnLength;
    // The length of a single element of the column type.

- (BOOL)defineWithCursor:(Cda_Def *)cda selectPosition:(unsigned)index 
    rowCapacity:(unsigned)capacity;
    // Called by the channel to allow the column to define the output buffers.

- (BOOL)canUseArrayFetching;
    // If all of the OracleColumns in a select list return YES, then the
    // channel will use array fetching to get more than one row at a time from
    // the server.

- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
    // This is the method which gets called to actually fetch data for a column
    // Returns a *retained* object (caller must release).

@end

// This is the column class from which all primitive column types descend.
// It implements the basics of talking to the client library to get data out
// of the database.

@interface OracleConcreteColumn:OracleColumn
{
    NSString *_attributeName;
    sb2 *_indicator;
    ub2 *_returnLength;
    ub2 *_returnCode;
    ub1 *_columnBuffer;
    
    OracleClientType _clientType;
    int _columnLength;
}

- initWithAttribute:(EOAttribute *)attribute;
    // Each column subclass must set _clientType and _columnLength in their
    // initWithAttribute: method.

- (int)columnLength;
    // This is a simple accessor for the _columnLength ivar.

- (BOOL)defineWithCursor:(Cda_Def *)cda selectPosition:(unsigned)index 
    rowCapacity:(unsigned)capacity;
    // Does the odefin () to set up the buffers for fetching data and getting
    // the various return codes.  The instance variables _clientType and
    // _columnLength must be set up by the subclasses before getting here.

- (BOOL)canUseArrayFetching;
    // By default this returns YES.  Subclasses should return NO if they cannot
    // use Oracle's array fetching API (if they need to use oflng ()).

@end

// These are the various kinds of primitive Oracle column buffers...

@interface OracleStringColumn:OracleConcreteColumn
- initWithAttribute:(EOAttribute *)attribute;
- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
@end

@interface OracleDataFromStringColumn:OracleStringColumn
- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
@end

@interface OracleNumberColumn:OracleConcreteColumn
- initWithAttribute:(EOAttribute *)attribute;
- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
@end

@interface OracleDataColumn:OracleConcreteColumn
- initWithAttribute:(EOAttribute *)attribute;
- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
@end

@interface OracleDateColumn:OracleConcreteColumn
{
    EOAttribute *_attribute;
}
- initWithAttribute:(EOAttribute *)attribute;
- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
@end

// While all the other kinds of columns are based on the value they are going
// to create, the OracleLongColumn is based on the value it has to fetch.
// Getting a LONG or LONG RAW column out of the database requires some special
// work.  It can return an NSString or an NSData.

@interface OracleLongColumn:OracleConcreteColumn
{
    Cda_Def *_cda;
    int _position;
    BOOL _objectIsData;
}
- initWithAttribute:(EOAttribute *)attribute;
- (BOOL)canUseArrayFetching;
- (BOOL)defineWithCursor:(Cda_Def *)cda selectPosition:(unsigned)position 
    rowCapacity:(unsigned)capacity;
- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
@end

// The OracleCustomColumn is the only non-primitive column type.  It holds
// another column which actually gets the data out of Oracle's buffers, and
// then creates a custom class from that primitive data.

@interface OracleCustomColumn:OracleColumn
{
    OracleColumn *_realColumn;
    NSString *_attributeName;
    NSString *_valueType;
    Class _valueClass;
    BOOL _primitiveValueIsData;
    BOOL _archivedObject;
    BOOL _oldArchive;
}
- initWithAttribute:(EOAttribute *)attribute;
- (int)columnLength;
- (BOOL)defineWithCursor:(Cda_Def *)cda selectPosition:(unsigned)index 
    rowCapacity:(unsigned)capacity;
- (BOOL)canUseArrayFetching;
- fetchFromIndex:(unsigned)index zone:(NSZone *)zone;
@end

